/*******************************************************************************

  Example Format Plugin Library (DLL) for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    12/05/2000 20:36:48 - First Creation
    03/25/2001 17:11:55 - Conversion to v2.1
    06/20/2004 01:10:00 - DIV plugin interface v5

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <magick/api.h>

#include <windows.h>

#include "div5_magick.h"
#include "magick/dim_magick_format.h"

TDivFormatItem *divFormatItems;

TDIVPluginHeader PluginHeader = {
  sizeof( TDIVPluginHeader ),
  5, // plugin interface version - MUST BE 5
  DIV_PLUGIN_TYPE_FORMAT,
  2, // plugin version
  1, // one format, this is basically not used for this type
  "DIMIN DIV5 Image Magick Formats"
};

//------------------------------------------------------------------------------
// About procedure
//------------------------------------------------------------------------------
void __stdcall aboutPlugin( TDivFormatHandle *fmtHndl, HWND hwndParent ) 
{
  MessageBox(GetFocus(), "DIMIN Image Magick Plug-In v1.0.3", "About Plug-in", MB_OK | MB_ICONEXCLAMATION);
}

//------------------------------------------------------------------------------
// Format definition
//------------------------------------------------------------------------------
#define MAGICK_NUMBER_FORMATS 84

TDivFormatItem divFormatItem[MAGICK_NUMBER_FORMATS] = {

{
  "ART",
  "PFS: 1st Publisher",
  "art",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "AVS",
  "AVS X image",
  "avs",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "CGM",
  "Computer Graphics Metafile",
  "cgm",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "CIN",
  "Kodak Cineon Image Format",
  "cin",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "CIP",
  "Cisco IP phone image format",
  "cip",
  0, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "CMYK",
  "Raw cyan, magenta, yellow, and black samples",
  "cmyk",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "CMYKA",
  "Raw cyan, magenta, yellow, black, and opacity samples",
  "cmyka",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "CUT",
  "DR Halo",
  "cut",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "DCM",
  "Digital Imaging and Communications in Medicine image",
  "dcm",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "DCX",
  "ZSoft IBM PC multi-page Paintbrush",
  "dcx",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "DOT",
  "Graph Visualization",
  "dot",
  0, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "DPS",
  "Display Postscript Interpreter",
  "dps",
  0, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "DPX",
  "Digital Moving Picture Exchange",
  "dpx",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EPI",
  "Encapsulated PostScript Interchange format",
  "epi|epsi",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EPS",
  "Encapsulated PostScript",
  "eps|epsf",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EPS2",
  "Level II Encapsulated PostScript",
  "EPS2",
  0, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EPS3",
  "Level III Encapsulated PostScript",
  "EPS3",
  0, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EPT",
  "Encapsulated PostScript with TIFF preview",
  "EPT",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EPT2",
  "Encapsulated PostScript Level II with TIFF preview",
  "EPT2",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EPT3",
  "Encapsulated PostScript Level III with TIFF preview",
  "EPT3",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "EXR",
  "High dynamic-range (HDR) file format",
  "exr",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "FIG",
  "FIG graphics format",
  "fig",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "FITS",
  "Flexible Image Transport System",
  "FITS",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "FPX",
  "FlashPix Format",
  "FPX",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "HDF",
  "Hierarchical Data Format",
  "HDF",
  0, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "JBIG",
  "Joint Bi-level Image experts Group interchange format",
  "jbg|jbig|bie",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "JNG",
  "Multiple-image Network Graphics",
  "jng",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MAP",
  "Colormap intensities and indices",
  "MAP",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MAT",
  "MATLAB image format",
  "MAT",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MIFF",
  "Magick Image File Format",
  "MIFF",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MNG",
  "Multiple-image Network Graphics",
  "MNG",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MPC",
  "Magick Persistent Cache image format",
  "MPC",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MSL",
  "Magick Scripting Language",
  "MSL",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MTV",
  "MTV Raytracing image format",
  "MTV",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "MVG",
  "Magick Vector Graphics",
  "MVG",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "OTB",
  "On-the-air bitmap",
  "OTB",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "P7",
  "Xv thumbnail format",
  "P7",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PAL",
  "16bit/pixel interleaved YUV",
  "pal|uyvy",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PALM",
  "Palm pixmap",
  "PALM",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PBM",
  "Portable bitmap format (black and white)",
  "PBM",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PCD",
  "Photo CD",
  "pcd|pcds",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PCL",
  "Page Control Language",
  "PCL",
  0, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PICT",
  "Apple Macintosh QuickDraw/PICT",
  "pct|pict",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PCX",
  "ZSoft IBM PC Paintbrush",
  "PCX",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PDB",
  "Palm Database ImageViewer Format",
  "PDB",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PFA",
  "Postscript Type 1 font (ASCII)",
  "PFA",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PFB",
  "Postscript Type 1 font (binary)",
  "PFB",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PGM",
  "Portable graymap format (gray scale)",
  "PGM",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PIX",
  "Alias/Wavefront RLE image format",
  "PIX",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PNM",
  "Portable anymap",
  "PNM",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PPM",
  "Portable pixmap format (color)",
  "PPM",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PS",
  "PostScript",
  "PS",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PS2",
  "Level II PostScript",
  "PS2",
  0, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PS3",
  "Level III PostScript",
  "PS3",
  0, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PSD",
  "Adobe Photoshop bitmap",
  "PSD",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "PWP",
  "Seattle Film Works",
  "pwp|sfw",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "RAD",
  "Radiance image file",
  "rad",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "RAS",
  "SUN Rasterfile",
  "ras|sun",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "RLA",
  "Alias/Wavefront image",
  "RLA",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "RLE",
  "Utah Run length encoded image",
  "RLE",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "SCR",
  "ZX-Spectrum SCREEN",
  "SCR",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "SCT",
  "Scitex Continuous Tone Picture",
  "SCT",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "SGI",
  "Irix RGB image",
  "SGI",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "STEGANO",
  "Steganographic image",
  "STEGANO",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "SVG",
  "Scalable Vector Graphics",
  "SVG",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "SVGZ",
  "Compressed Scalable Vector Graphics",
  "SVGZ",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "TGA",
  "Truevision Targa image",
  "tga|icb|vda|vst",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "TIM",
  "PSX TIM",
  "TIM",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "TTC",
  "TrueType font collection",
  "TTC",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "TTF",
  "TrueType font",
  "TTF",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "UIL",
  "X-Motif UIL table",
  "UIL",
  0, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "VICAR",
  "VICAR rasterfile format",
  "VICAR",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "VID",
  "Visual Image Directory",
  "VID",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "VIFF",
  "Khoros Visualization image",
  "viff|xv",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "WBMP",
  "Wireless Bitmap (level 0) image",
  "WBMP",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "WPG",
  "Word Perfect Graphics",
  "WPG",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "XBM",
  "X Windows system bitmap (black and white)",
  "XBM",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "XC",
  "Constant image uniform color",
  "XC",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "XCF",
  "GIMP image",
  "XCF",
  1, //canRead;      // 0 - NO, 1 - YES
  0, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "XPM",
  "X Windows system pixmap (color)",
  "XPM",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "XWD",
  "X Windows system window dump (color)",
  "XWD",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "YCbCr",
  "Raw Y, Cb, and Cr samples",
  "YCbCr",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "YCbCrA",
  "Raw Y, Cb, Cr, and opacity samples",
  "YCbCrA",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
},
{
  "YUV",
  "CCIR 601 4:1:1 or 4:2:2",
  "YUV",
  1, //canRead;      // 0 - NO, 1 - YES
  1, //canWrite;     // 0 - NO, 1 - YES
  0, //canReadMeta;  // 0 - NO, 1 - YES
  0, //canWriteMeta; // 0 - NO, 1 - YES
  0, //canAddMeta;   // 0 - NO, 1 - YES
  { 0, 0, 0, 0, 0, 0, 0, true }
}

};

TDivFormatHeader formatHeader = {
  sizeof(TDivFormatHeader),      // must be == sizeof(TDivFormatHeader)
  "1.0.3",                  // plugin version, e.g. "1.0.0"
  "ImageMagick Codec",       // plugin name
  "ImageMagick codec for div5",    // plugin description
  
  50, //8, // 0 or more, specify number of bytes needed to identify the file // 8 - all, 56 - BioRad
  {1, MAGICK_NUMBER_FORMATS, divFormatItem}, //TDivFormatList              supportedFormats;
  
  dimMagickValidateFormatProc, //TDivValidateFormatProc      validateFormatProc; // return TRUE if can handle file
  // begin
  dimMagickAquireFormatProc, //TDivAquireFormatProc        aquireFormatProc;
  // end
  dimMagickReleaseFormatProc, //TDivReleaseFormatProc       releaseFormatProc;

  // params
  aboutPlugin, //TDivAboutFormatProc         aboutFormatProc;
  NULL, //TDivConfigFormatProc        configFormatProc;

  // image begin
  dimMagickOpenImageProc, //TDivOpenImageProc           openImageProc; 
  dimMagickCloseImageProc, //TDivCloseImageProc          closeImageProc; 

  // info
  dimMagickGetNumPagesProc, //TDivGetNumPagesProc         getNumPagesProc;
  dimMagickGetImageInfoProc, //TDivGetImageInfoProc        getImageInfoProc;

  // read/write
  dimMagickReadImageProc, //TDivReadImageProc           readImageProc; 
  dimMagickWriteImageProc, //TDivWriteImageProc          writeImageProc;
  NULL, //TDivReadImageTileProc       readImageTileProc;
  NULL, //TDivWriteImageTileProc      writeImageTileProc;
  NULL, //TDivReadImageLineProc       readImageLineProc;
  NULL, //TDivWriteImageLineProc      writeImageLineProc;
  NULL, //TDivReadImageThumbProc      readImageThumbProc;
  NULL, //TDivWriteImageThumbProc     writeImageThumbProc;
  NULL, //TDivReadImagePreviewProc    readImagePreviewProc;
  
  // meta data
  NULL, //TDivReadMetaDataProc        readMetaDataProc;
  NULL, //TDivAddMetaDataProc         addMetaDataProc;
  dimMagickReadMetaDataAsTextProc, //TDivReadMetaDataAsTextProc  readMetaDataAsTextProc;

  NULL, //dimMagickConfigInitProc, //TDivConfigFormatProc
  NULL, //dimMagickConfigQuitProc, //TDivConfigFormatProc

  // xtd
  NULL, // xtdProc

  NULL, // p1
  NULL, // p2
  NULL, // p3

};

//---------------------------------------------------------------------------

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
  switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


TDIVPluginHeader* __stdcall divGetPluginHeader()
{
   return &PluginHeader;
}

TDivFormatHeader* __stdcall divGetFormatHeader()
{
 /*
  int i, num=0; 
  MagickInfo **ia;
  unsigned long number_formats;

  try
  {
    InitializeMagick(".");  
    ia = (MagickInfo **) GetMagickInfoList( "", &number_formats, NULL );

    divFormatItems = new TDivFormatItem [ number_formats ];

    // now create format list
    for (i=0; i<number_formats; i++)
    {
      divFormatItems[i] = divFormatItem[0];
      if (ia[i]->decoder != NULL) divFormatItems[i].canRead = 1; else divFormatItems[i].canRead = 0;
      if (ia[i]->encoder != NULL) divFormatItems[i].canWrite = 1; else divFormatItems[i].canWrite = 0;
      divFormatItems[i].formatNameShort = ia[i]->name;
      divFormatItems[i].extensions      = ia[i]->name;
      divFormatItems[i].formatNameLong  = ia[i]->description;
      num++;
    }

    formatHeader.supportedFormats.count = number_formats;
    formatHeader.supportedFormats.item  = divFormatItems;
  }
  catch ( ... )
  {

  }
*/
  return &formatHeader;
}



//---------------------------------------------------------------------------
