/*****************************************************************************
  Digital Cameras RAW file support
  Copyright (c) 2004 by Dmitry V. Fedorov <www.dimin.net> <dima@dimin.net>

  IMPLEMENTATION
  
  Programmer: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>

  Note:
    Now supports Big-Endian and Small-Endian arcitectures

  History:
    10/26/2004 19:00 - First creation
        
  Ver : 1
*****************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "dim_magick_format.h"

//#include <Magick++.h>
//using namespace Magick;  

#include <magick/api.h>
#include <magick/nt-feature.h>

TDivFormatHandle *gFmtHndl;

unsigned char ms_thumbs_db[8] = { 0xD0, 0xCF, 0x11, 0xE0, 0xA1, 0xB1, 0x1A, 0xE1 };


//****************************************************************************
// MISC
//****************************************************************************

TDimMagickParams initTDimMagickParams()
{
  TDimMagickParams r;

  r.i = initTDivImageInfo();
  r.i.depth  = 8;

  return r;
}


//****************************************************************************
//
// FORMAT DEMANDED FUNTIONS
//
//****************************************************************************

//----------------------------------------------------------------------------
// PARAMETERS, INITS
//----------------------------------------------------------------------------

DIM_INT __stdcall dimMagickValidateFormatProc (void *magic, DIM_UINT32 length)
{
  char *magick_str;
  //if (memcmp(magic,"%!",2) == 0) return 0;
  //if (memcmp(magic,"\004%!",3) == 0) return 0;

  if ( memcmp(magic, ms_thumbs_db, 8) == 0) return -1;

  magick_str = (char *) GetImageMagick( (unsigned char *) magic, length );
  if ( magick_str == NULL) return -1;

  // do not read internally supported formats
  if ( strncmp( magick_str, "BMP", 3 )  == 0) return -1;
  //if ( strncmp( magick_str, "BMP2", 4 )  == 0) return -1;
  //if ( strncmp( magick_str, "BMP3", 4 )  == 0) return -1;  
  if ( strncmp( magick_str, "DIB", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "EMF", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "WMF", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "WMZ", 3 )  == 0) return -1;
  //if ( strncmp( magick_str, "WMFWIN32", 8 )  == 0) return -1;
  
  if ( strncmp( magick_str, "GIF", 3 )  == 0) return -1;
  //if ( strncmp( magick_str, "GIF87", 5 )  == 0) return -1;

  if ( strncmp( magick_str, "JP2", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "PGX", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "JPC", 3 )  == 0) return -1;
  
  if ( strncmp( magick_str, "JPEG", 4 ) == 0) return -1;
  if ( strncmp( magick_str, "JPG", 3 ) == 0) return -1;

  if ( strncmp( magick_str, "MNG", 3 ) == 0) return -1;
  if ( strncmp( magick_str, "PNG", 3 ) == 0) return -1;
  if ( strncmp( magick_str, "JNG", 3 ) == 0)return -1;
  //if ( strncmp( magick_str, "PNG8", 4 ) == 0) return -1;
  //if ( strncmp( magick_str, "PNG24", 5 ) == 0) return -1;
  //if ( strncmp( magick_str, "PNG32", 5 ) == 0) return -1;

  if ( strncmp( magick_str, "TIF", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "PTIF", 4 )  == 0) return -1;
  if ( strncmp( magick_str, "TIFF", 4 ) == 0) return -1;
  if ( strncmp( magick_str, "FAX", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "G3", 2 )  == 0) return -1;

  if ( strncmp( magick_str, "RLA", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "RGB", 3 )  == 0) return -1;
  if ( strncmp( magick_str, "RGBA", 4 ) == 0) return -1;

  if ( strcmp( magick_str, "R" ) == 0) return -1;
  if ( strcmp( magick_str, "C" ) == 0) return -1;
  if ( strcmp( magick_str, "G" ) == 0) return -1;
  if ( strcmp( magick_str, "M" ) == 0) return -1;
  if ( strcmp( magick_str, "B" ) == 0) return -1;
  if ( strcmp( magick_str, "Y" ) == 0) return -1;
  if ( strcmp( magick_str, "A" ) == 0) return -1;
  if ( strcmp( magick_str, "O" ) == 0) return -1;
  if ( strcmp( magick_str, "K" ) == 0) return -1;


  if ( magick_str != NULL) return 0;

  return -1;
}

TDivFormatHandle __stdcall dimMagickAquireFormatProc( void )
{
  TDivFormatHandle fp = initTDivFormatHandle();
  fp.quality = 1;
  //InitializeMagick(".");
  MagickCoreGenesis(NULL, MagickBooleanType::MagickFalse);
  return fp;
}

void __stdcall dimMagickReleaseFormatProc (TDivFormatHandle *fmtHndl)
{
  if (fmtHndl == NULL) return;
  try
  {
    dimMagickCloseImageProc ( fmtHndl );
    //DestroyMagick(); 
    MagickCoreTerminus();
  }
  catch (...)
  {

  }
}


//----------------------------------------------------------------------------
// OPEN/CLOSE
//----------------------------------------------------------------------------
void __stdcall dimMagickCloseImageProc (TDivFormatHandle *fmtHndl)
{
  if (fmtHndl == NULL) return;
  dimClose ( fmtHndl );
  //dimFree ( &fmtHndl->internalParams );
  if (fmtHndl->internalParams != NULL) delete fmtHndl->internalParams;
  fmtHndl->internalParams = NULL;
}

DIM_UINT __stdcall dimMagickOpenImageProc  (TDivFormatHandle *fmtHndl, TDIVIOModes io_mode)
{
  if (fmtHndl == NULL) return 1;
  if (fmtHndl->internalParams != NULL) dimMagickCloseImageProc (fmtHndl);  
  fmtHndl->internalParams = (void *) new TDimMagickParams [1];
  TDimMagickParams *picPar = (TDimMagickParams *) fmtHndl->internalParams;
  *picPar = initTDimMagickParams();

  //if (io_mode == DIV_IO_READ) return 0;
  //else return 1;

  return 0;
}

//----------------------------------------------------------------------------
// INFO for OPEN image
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimMagickGetNumPagesProc ( TDivFormatHandle *fmtHndl )
{
  if (fmtHndl == NULL) return 0;
  if (fmtHndl->internalParams == NULL) return 0;
  TDimMagickParams *picPar = (TDimMagickParams *) fmtHndl->internalParams;

  //return picPar->i.number_pages;
  return 1;
}


TDivImageInfo __stdcall dimMagickGetImageInfoProc ( TDivFormatHandle *fmtHndl, DIM_UINT page_num )
{
  TDivImageInfo ii = initTDivImageInfo();

  if (fmtHndl == NULL) return ii;
  TDimMagickParams *picPar = (TDimMagickParams *) fmtHndl->internalParams;

  return picPar->i;
}

//----------------------------------------------------------------------------
// METADATA
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimMagickReadMetaDataProc (TDivFormatHandle *fmtHndl, DIM_UINT page, int group, int tag, int type)
{
  if (fmtHndl == NULL) return 1;
  //return read_biorad_metadata (fmtHndl, group, tag, type);
  //return 0;
  return 1;
}

char* __stdcall dimMagickReadMetaDataAsTextProc ( TDivFormatHandle *fmtHndl )
{
  if (fmtHndl == NULL) return NULL;
  //return read_text_biorad_metadata ( fmtHndl );
  //return 0;
  return NULL;
}


//----------------------------------------------------------------------------
// READ/WRITE
//----------------------------------------------------------------------------


MagickBooleanType mProgressMonitor(const char *str, const MagickOffsetType offset, const MagickSizeType size, void *ptr )
{
  showProgress ( gFmtHndl, offset, size, (char *) str );
  if ( testAbort( gFmtHndl ) == 1) return MagickFalse;
  return MagickTrue;
}

DIM_UINT __stdcall dimMagickReadImageProc  ( TDivFormatHandle *fmtHndl, DIM_UINT page )
{
  if (fmtHndl == NULL) return 1;
  if (fmtHndl->internalParams == NULL) return 1;  
  TDimMagickParams *picPar = (TDimMagickParams *) fmtHndl->internalParams;
  fmtHndl->pageNumber = page;

  long w, h, depth, x;
  gFmtHndl = fmtHndl;
  ExceptionInfo *exception;
  Image *image;
  ImageInfo *image_info;

  try {
    exception = AcquireExceptionInfo();
    image_info = CloneImageInfo((ImageInfo *) NULL);
    (void) strcpy(image_info->filename, fmtHndl->fileName );
    image_info->progress_monitor = mProgressMonitor;

    //image = PingImage(image_info, exception);
    image = ReadImage( image_info, exception );
  
    if (exception->severity != UndefinedException) {
      //CatchException(&exception);
      showError( fmtHndl, 0, exception->description );
      return 1;
    }
    if (image == (Image *) NULL) return 1;

    if ( image->colors != 256 )
      fmtHndl->hbmp = (HBITMAP) ImageToHBITMAP( image );
    else {
      if ( image->colorspace != GRAYColorspace ) SetImageColorspace( image, RGBColorspace );

      int num_colors = image->colors;
      depth = min( image->depth, 8);
      h = image->rows;
      w = image->columns;
      unsigned char *p;
      RGBQUAD lut[256];

      //-------------------------------------------------------------------
      // define palette
      //-------------------------------------------------------------------
      int c=0, inc=1, i;

      if (num_colors == 2) inc=255;
      else
      if (num_colors == 4) inc=85;
      else
      if (num_colors == 16) inc=16;
  
      for (i=0; i<num_colors; i++) {
        lut[i].rgbRed   = c;
        lut[i].rgbGreen = c;
        lut[i].rgbBlue  = c;   
        c+=inc; 
      }

      // read palette if paletted
      if (IsPaletteImage ( image, exception ) == MagickTrue) {
        PixelPacket entry;
        for (i=0; i<num_colors; i++) {
          entry = image->colormap[i];
          lut[i].rgbRed   = ScaleQuantumToChar(entry.red);
          lut[i].rgbGreen = ScaleQuantumToChar(entry.green);
          lut[i].rgbBlue  = ScaleQuantumToChar(entry.blue);   
        }       
      } 

      //-------------------------------------------------------------------
      // create image
      //-------------------------------------------------------------------
      if (dimCreateHBitmap( fmtHndl, w, h, depth, num_colors, lut, &p ) == 0) {
        const PixelPacket *pPixels;
        unsigned long row;
        unsigned char *pix;

        for (row = 0 ; row <image->rows ; row++ ) {
          pPixels = AcquireImagePixels( image, 0, row, image->columns, 1, &image->exception );
          unsigned long nPixelCount;

          showProgress( fmtHndl, row, image->rows, "Reading Magick" );
          if ( testAbort( fmtHndl ) == 1) break;  
          pix = getHBScanLine( image->rows-row-1, p, w, depth );

          // Transfer pixels, scaling to Quantum
          x = 0;
          for( nPixelCount = image->columns; nPixelCount ; nPixelCount-- ) {
            pix[x] = Downscale(pPixels->red);
            ++pPixels;
            x++;   
          } // for
        } // for row
      } // if bitmap 
    } // if colors != 0

    image = DestroyImage( image ); 
    image_info = DestroyImageInfo( image_info );
    exception = DestroyExceptionInfo(exception);
    return 0; 
  } 
  catch( ... )
  {
    return 1;
  }

  return 0;
}

DIM_UINT __stdcall dimMagickWriteImageProc ( TDivFormatHandle *fmtHndl ) {
  gFmtHndl = fmtHndl;
  ExceptionInfo exception;
  Image *image;
  ImageInfo *image_info;

  try
  {
    BITMAP bitmap;
    // Retrieve the bitmap color format, width, and height.
    GetObject( fmtHndl->hbmp, sizeof(BITMAP), (LPSTR)&bitmap );
    unsigned char *p = (unsigned char *) bitmap.bmBits;
  
    GetExceptionInfo(&exception);
    image_info = CloneImageInfo((ImageInfo *) NULL);
    image_info->progress_monitor = mProgressMonitor;
    
    if (bitmap.bmBitsPixel == 8) 
    {  
      image_info->colors = 256;
      image_info->colorspace = GRAYColorspace;
      image_info->type = GrayscaleType;
      image_info->depth = 8;
    }
    else 
    {
      image_info->colors = 0;
      image_info->colorspace = RGBColorspace;
      image_info->type = TrueColorType;
      image_info->depth = 8;
    }

    long y;
    register long x;
    register PixelPacket *q;

    image = AllocateImage(image_info);
    image->rows = bitmap.bmHeight;
    image->columns = bitmap.bmWidth;

    if (bitmap.bmBitsPixel == 8) 
    {  
      image->colors = 256;
      image->depth = 8;
      SetImageColorspace( image, GRAYColorspace );   
    }

    // finally copy the pixels!
    for (y=0; y < (long) image->rows; y++)
    {
      showProgress( fmtHndl, y, image->rows, "Writing magick image" );
      if ( testAbort( fmtHndl ) == 1) break;        

      q = SetImagePixels( image, 0, y, image->columns, 1 );
      if (q == (PixelPacket *) NULL) break;
 
      if (bitmap.bmBitsPixel == 8)
      {
        int hbmpy = image->rows-y-1;  
        unsigned char *pix = getHBScanLine( hbmpy, p, bitmap.bmWidth, bitmap.bmBitsPixel );

        for (x=0; x < (long) image->columns; x++)
        {
          q->red     = Downscale( pix[0] );
          q->blue    = Downscale( pix[0] );
          q->green   = Downscale( pix[0] );
          q->opacity = OpaqueOpacity;
          pix++;
          q++;
        }
      } // if one sample with 8 bits
      else
      {
        int hbmpy = image->rows-y-1;  
        unsigned char *pix = getHBScanLine( hbmpy, p, bitmap.bmWidth, bitmap.bmBitsPixel );

        for (x=0; x < (long) image->columns; x++)
        {
          q->red     = Downscale( pix[2] );
          q->blue    = Downscale( pix[1] );
          q->green   = Downscale( pix[0] );
          q->opacity = OpaqueOpacity;
          pix+=3;
          q++;
        }
      } // if not gray

      if (!SyncImagePixels(image)) break;
    }

    (void) strcpy(image_info->filename, fmtHndl->fileName );
    (void) strcpy(image->filename, fmtHndl->fileName );

    image_info->progress_monitor = mProgressMonitor;
    image->progress_monitor = mProgressMonitor;

    WriteImage( image_info, image );
    DestroyImageList( image );
    DestroyImageInfo( image_info );
    DestroyExceptionInfo( &exception );
    return 0;
  }
  catch ( ... )
  {
    return 1;
  }
}

